import {
  select, 
  csv, 
  scaleLinear, 
  max,
  scaleBand,
  axisLeft,
  axisBottom,
  format
} from 'd3';

const svg = select('svg');
const height = +svg.attr('height');
const width = +svg.attr('width');

const title = 'Top 10 populous countries';
const xAxisLabel = 'Population';
const yAxisLabel = 'Country';
const direction = 'horizontal';

const xGrid = 1;
const yGrid = 1;
const xGridColor = '#C0C0BB';
const yGridColor = '#C0C0BB';
const xGridStrokeWidth = 1;
const yGridStrokeWidth = 1;

const margin = {top: 60, right: 40, bottom: 70, left: 200};
const innerWidth = width - margin.left - margin.right;
const innerHeight = height - margin.top - margin.bottom;


const gridCreator = (theAxis) => {

  if(theAxis == 'x' && xGrid == 1) {
  	return -innerHeight;
  } else if (theAxis == 'y' && yGrid == 1) {
  	return -innerWidth;
  }
  
  return 0;
}
  



const render = data => {
  
  
  //Value accessors. Limits the specifity of code to xValue and yValue
  const xValue = direction == 'horizontal'? d => d.value : d => d.label;
 	const yValue = direction == 'horizontal'? d => d.label: d => d.value;
 	
 
  const setScales = () => {
			if(direction == 'horizontal') {
				return setHorizontalScales();
			} else {
				return setVerticalScales();
			}
		}
	
  const setHorizontalScales = () => {
  //Maps values from domain to corresponding values in range(Data Space to Screen Space)
  const xScaleEval = scaleLinear()
  	.domain([0, max(data, xValue)])
  	.range([0, innerWidth])
  	.nice(); //Smooth edging of axes
  
  //Separate the bars and determine their heights
  const yScaleEval = scaleBand()
  	.domain(data.map(yValue))
  	.range([0, innerHeight])
  	.padding(0.1);
  
    return {yScale: yScaleEval, xScale: xScaleEval};
    
  }
  
   const setVerticalScales = () => {
  //Maps values from domain to corresponding values in range(Data Space to Screen Space)
  const yScaleEval = scaleLinear()
  	.domain([max(data, yValue),0])
  	.range([0, innerHeight])
  	.nice(); //Smooth edging of axes
  
  //Separate the bars and determine their heights
  const xScaleEval = scaleBand()
  	.domain(data.map(xValue))
  	.range([0, innerWidth])
  	.padding(0.1);
  
    return {yScale: yScaleEval, xScale: xScaleEval};
    
  }
  
  
  
   const xScale = setScales().xScale;
   const yScale = setScales().yScale;
    
 
  
  //Formatting numbers
  const theAxisTickFormat = number =>
  	format('.3s')(number)
  		.replace('G', 'B');
  
  //The axes space
  const g = svg.append('g')
  	.attr('transform', 'translate('+margin.left+', '+margin.top+')');

  
  const xAxis =  axisBottom(xScale)
  	//The vertical grid
  	//.ticks(20) //Need to look into this method
  	.tickSize(gridCreator('x'))
  	.tickPadding(10); //Label distance from X axis

  const yAxis = axisLeft(yScale)
  	//The horizontal grid
  	.tickSize(gridCreator('y'))
  	.tickPadding(10); //Label distance from Y axis;
  
   direction == 'horizontal'? xAxis.tickFormat(theAxisTickFormat) : yAxis.tickFormat(theAxisTickFormat);
 
 
  const yAxisG = g.append('g').call(yAxis);
  
  yAxisG.selectAll('.domain').remove();
  
  
  //Y Axis Label
	yAxisG.append('text')
  	.attr('y', -110)
  	.attr('class', 'axis-label')
  	.attr('x', -innerHeight/2)
  	.attr('fill', 'black')
  	.attr('transform', 'rotate(-90)')
  	.attr('text-anchor', 'middle')
  	.text(yAxisLabel);
  
  //Y Grid Color and Stroke Width
  yAxisG.selectAll('g.tick')
  	.select('line')
  	.style('stroke', yGridColor)
  	.style('stroke-width', yGridStrokeWidth);
  
  //Call to X axis
  const xAxisG = g.append('g').call(xAxis)
  	.attr('transform', 'translate(0, '+innerHeight+')');
  
  // Remove lines and ticks(not ticks in this)
  xAxisG.select('.domain').remove();
  
  //X Axis Label
  xAxisG.append('text')
  	.attr('y', 60)
  	.attr('class', 'axis-label')
  	.attr('x', innerWidth/2)
  	.attr('fill', 'black')
  	.text(xAxisLabel);
  
  //X Grid Color and Stroke Width
  xAxisG.selectAll('g.tick')
  	.select('line')
  	.style('stroke', xGridColor)
  	.style('stroke-width', xGridStrokeWidth);
  
   const createBar = () => {
			if(direction == 'horizontal') {
				createHorizontalBar();
			} else {
				createVerticalBar();
			}
		}
 	
  
  const createHorizontalBar = () => {
    
	g.selectAll('rect').data(data)
  	.enter().append('rect')
  	.attr('y', d => yScale(yValue(d)))
  	.attr('width', d => xScale(xValue(d)))
    //.attr('fill', d => d.color)
  	.attr('height', yScale.bandwidth());
    
  }
  
   const createVerticalBar = () => {
    
	g.selectAll('rect').data(data)
  	.enter().append('rect')
  	.attr('x', d=> xScale(xValue(d)))
  	.attr('y', d => yScale(yValue(d)))
  	.attr('height', d => innerHeight-yScale(yValue(d)))
  	.attr('width', xScale.bandwidth())
  }
   
  
  
  createBar();
  g.append('text')
  	.attr('class', 'title')
  	.attr('y', -10)
  	.text(title);
};

//Fetch the csv and calls the render() method
//Returns a promise that resolves when data is loaded
csv('data.csv').then(data => { 
  data.forEach(d => {
  d.value = +d.value*1000;
  });
 render(data)
});

